######################################################################
# Copyright 1995-1999 Nokia IP, Inc.  All rights reserved.
#
# YOU MAY ONLY USE THE MATERIALS FOUND IN THIS FILE PURSUANT TO THE
# TERMS OF THE EVALUATION LICENSE AGREEMENT FOUND IN "/LICENSE".  ALL
# OTHER USES ARE EXPRESSLY PROHIBITED.
######################################################################
#
# $Id: validate.tcl,v 1.57.44.2 2007/01/31 00:11:41 skimble Exp $
#

# Some utility functions for backend validation
# Include this file in your tcl script by the following command
# 	source validate.tcl
# This inclusion can be done anywhere in the script except from inside a 
# block or procedure

package require Mpexpr

#
# Get logical interfaces belonging to a given routing instance (including those 
# which are not associated with any instance) 
# param: db - db handle, instance - instance name, family - ipv4 or ipv6
# db_slot - for sequoia (node:$slot_num:)
#
proc get_interfaces {db instance family {db_slot ""}} {
   if {$instance == ""} {
      set instance "default"
   }
   if {$family == "ipv4"} {
      set family "inet"
   } elseif {$family == "ipv6"} {
      set family "inet6"
   }
   set intfs_list [list]
   set intfs [libdb iterate -c on -n on $db "${db_slot}interface"]
   if {[libdb get $db "xpand:instance"] == ""} {
      return $intfs
   }
   foreach i $intfs {
      set rtg [libdb get $db "${db_slot}interface:${i}:family:${family}:instance"]
      if { ($rtg == $instance) || ($rtg == "") } {
         lappend intfs_list $i
      }
   }
   return $intfs_list
}

#
# Get logical interfaces belonging to a given routing instance which also
# have address assigned to it - called by voyager routing pages and routing CLI.
# param: db - db handle, instance - instance name, family - ipv4 or ipv6
# db_slot - for sequoia "node:$slot_num:"
#
proc rtg_get_interfaces {db instance family {db_slot ""}} {
   if {$instance == ""} {
      set instance "default"
   }
   if {$family == "ipv4"} {
      set family "inet"
   } elseif {$family == "ipv6"} {
      set family "inet6"
   }
   set intfs_list [list]
   set intfs [libdb iterate -c on -n on $db "${db_slot}interface"]
   if {[libdb get $db "xpand:instance"] == ""} {
      return $intfs
   }

   foreach i $intfs {
      set rtg [libdb get $db "${db_slot}interface:${i}:family:${family}:instance"]
      if { ($rtg == $instance) || ($rtg == "") } {
         set addr_list ""
         if {$family == "inet"} {
            set addr_list [libdb iterate $db -c on "${db_slot}interface:${i}:ipaddr"]
            if {$addr_list == ""} {
               set addr_list [libdb get $db "${db_slot}interface:${i}:ipaddr"]
               if {$addr_list == ""} {
                  set addr_list [libdb get $db "${db_slot}interface:${i}:unnumbered"]
               }
            }
         } elseif {$family == "inet6"} {
            set addr_list [libdb get $db "${db_slot}interface:${i}:ip6"]
            if {$addr_list == ""} {
               set addr_list [libdb iterate $db -c on "${db_slot}interface:${i}:ip6addr"]
               if {$addr_list == ""} {
                  set addr_list [libdb get $db "${db_slot}interface:${i}:ip6addr"]
               }
            }
         }
         if {$addr_list != ""} {
            lappend intfs_list $i
         }
      }
   }
   return $intfs_list
}

#
# Takes n arguments and build url argument list containing non-empty args, i.e.
# build_arg_list a=b "" c=d returns a=b&c=d
#
proc build_arg_list {args} {
    set result ""
    set amb 0
    foreach item ${args} {
	if { [string length ${item}] > 0 } {
	    if { ${amb} != 0 } {
		append result "&"
	    }
	    append result ${item}
	    set amb 1
	}
    }
    return ${result}
}

# 
# Takes an input as an integer and assumes that it's a decimal integer
# removes any leading zeros and returns the number
#
proc stripzeros {value} {
    set retval [string trimleft $value 0]
    if { ![string length $retval] } {
        return 0
    }
    return $retval
}

# Check if input is a letter
proc checkLetter { c } {
  set ret [ regexp {^[a-zA-Z]$} $c ]
  return $ret
}

# Check if input is a digit
proc checkDigit { int } {
  set ret [ regexp {^[0-9]$} $int ]
  return  $ret
}

# Added this due to PR 10045
# Check if input is valid hex.  
proc checkHex { input } {
   set ret [string is xdigit -fail var $input]
   return $ret 
}


# Check if input is a letter or Digit
proc checkLetterOrDigit { str } {
  set ret [ checkLetter $str ]
  if { $ret == 0 } {
    set ret1 [ checkDigit $str ] 
    return $ret1
  } else {
     return $ret
  }
}

#
# Check if input matches user given regexp
# Returns:  0 if false
#           1 if true
#
proc checkUserDef { input regexp } {
    return [regexp $regexp $input]
}


#
# Check if input is an integer
# Returns:  0 if false
#           1 if true
#
proc checkInteger { input } {
  set ret [regexp {^(\+|-)?[0-9]+$} $input]
  if {$ret != 0} {
     set value [string trim $input]
     if {[string index $value 0] == "-"} {
         set value [string range $value 1 end]
         if {[mpexpr $value <= 2147483647]} {
             return 1
         } else {
             return 0
         }
     } elseif {[string index $value 0] == "+"} {
         set value [string range $value 1 end]
     }
     if {![catch {mpexpr $value > 0}]} {
         if {[mpexpr $value <= 4294967295]} {
             return 1
         } else {
             return 0
         }
     } else {
         return 0
     }
  }
  return $ret
}

# Check if input is a signed integer
proc checkSignedInteger { input } {
  return [checkInteger $input]
}


# Check if input is a positive integer
proc checkPositiveInteger { input } {
   set ret [checkSignedInteger $input]
   if {$ret} {
       if {$input == 0 || [string index [string trim $input] 0] == "-"} {
           set ret 0
       }
   }
   return $ret
}


# Check if input is non-negative integer
proc checkNonnegativeInteger { input } {
   set ret [ checkSignedInteger $input ]
   if {$ret && [string index [string trim $input] 0] == "-" && $input != 0} {
     set ret 0
   }
   return $ret
}

# Check if input is negative integer 
proc checkNegativeInteger { input } {
   set ret [ checkSignedInteger $input ]
   if {$ret == 1 && [string index [string trim $input] 0] != "-" } {
      set ret 0
   }
   return $ret
}

# Check if input is non-positive integer
proc checkNonpositiveInteger { input } {
   set ret [ checkSignedInteger $input ]
   if { $ret == 1 } {
       if {[string index [string trim $input] 0] == "-" || $input == 0} {
       } else {
           set ret 0
       }
   }
   return $ret
}

# Check if input is a float
proc checkFloat { input } {
   set ret [ regexp {^(([0-9]+(\.[0-9]*)?)|(([0-9]*\.)?[0-9]+))$} $input ]
   return $ret
}

# Check if input is signed float
proc checkSignedFloat { input } {
   set ret [ regexp {^(((\+|-)?\d+(\.\d*)?)|((\+|-)?(\d*\.)?\d+))$} $input ]
   return $ret
}

# Check if input is alphabetic charachters
proc checkAlphabetic { input } {
   set ret [ regexp {^[a-zA-Z]+$} $input ]
   return $ret
}

# Check if input is alpha-numeric
proc checkAlphanumeric { input } {
   set ret [ regexp {^[a-zA-Z0-9]+$} $input ]
   return $ret        
}

# Check if input is a FileName
#proc checkFileName { input } {
#   set ret [ regexp {^[a-zA-Z0-9_\/\.]+$} $input ]
#   return $ret
#}

# Check if input is a HTTP, FTP, TELNET or FILE URL
proc checkURL { input } {
set ret [ regexp {^((http:\/\/)|(ftp:\/\/)|(telnet:\/\/)|(file:\/)|(file:\/\/[\/]+)|(ldap:\/\/))[a-zA-Z0-9][A-Za-z0-9_\.\,\/\~\:\%\?\=\#\&\@\+\\-]+$} $input ]
   return $ret
}

#######################################################
# Check if input is a hostname
# a) hostname begins with a letter or number and may contains only letters, 
#    numbers and a dot and a dash
# b) It may not end with a dot or a dash
# c) only a letter or a number may follow a dash
# d) Hostname length limit 63 chars (RFC 1123)
#    According to the discussions held with NMS folks
#    A valid hostanme has maximum length of 255 characters.
#    Each DOT separated text can have maximum 64 characters.
#    The above are based on
#    a) Defnition of MAXHOSTNAMELEN in src/sys/sys/param.h
#    b) RFC 
#
#<RFC_2181>
#    11. Name syntax
#
#   Occasionally it is assumed that the Domain Name System serves only
#   the purpose of mapping Internet host names to data, and mapping
#   Internet addresses to host names.  This is not correct, the DNS is a
#   general (if somewhat limited) hierarchical database, and can store
#   almost any kind of data, for almost any purpose.
#
#   The DNS itself places only one restriction on the particular labels
#   that can be used to identify resource records.  That one restriction
#   relates to the length of the label and the full name.  The length of
#   any one label is limited to between 1 and 63 octets.  A full domain
#   name is limited to 255 octets (including the separators) ...
#</RFC_2181>
# 
#######################################################
proc checkHostName { input } {

  if {([string length $input] > 255)} {
      return 0
  }
  set ret [ regexp {^[a-zA-Z0-9]([a-zA-Z0-9\.-])+$} $input ]
  if {$ret == 0} {
    return 0
  }
  set c [string index $input end]
  if {$c == "." || $c == "-"} {
    return 0
  }
  if {([string first ".-" $input] != -1) ||
           ([string first "-." $input] != -1)} {
      return 0
  }
  
  set l [split $input .]
  set len [llength $l]
  for {set i 0} {$i < $len} {incr i} {
    set substr [lindex $l $i]
    if {$substr == ""} {
       return 0
    }
    if {[string index $substr 0] == "-"} {
       return 0
    }
  }
  return 1
}

# Check if input is an email 
proc checkEmail { input } {
    set ret [ regexp {(^[a-zA-Z0-9][a-zA-Z0-9_\.-]*$)} $input ]
    if {$ret == 1} {
        return $ret
    }
    set ret [ regexp {(^[a-zA-Z0-9][a-zA-Z0-9_\.-]*\@[a-zA-Z0-9][a-zA-Z0-9_\.-]+$)} $input ]
    if {$ret == 0} {
        return $ret
    }
    set indx [string first "@" $input]
    set indx [expr $indx + 1]
    set sub_str [string range $input $indx end]
    return [checkHostName $sub_str]
}

# Check if input is a MAC address
proc checkMACAddr { input } {
set ret [ regexp {^[a-fA-F0-9]([a-fA-F0-9])?:[a-fA-F0-9]([a-fA-F0-9])?:[a-fA-F0-9]([a-fA-F0-9])?:[a-fA-F0-9]([a-fA-F0-9])?:[a-fA-F0-9]([a-fA-F0-9])?:[a-fA-F0-9]([a-fA-F0-9])?$} $input ]
   return $ret
}


# Check if input is a given integer is in range
proc checkIntegerInRange { input begin end } {
    if {[regexp {^-[0-9]+$} $begin] || [regexp {^-[0-9]+$} $end]} {
	return [checkSignedIntegerInRange $input $begin $end]
    } else {
	return [checkUnsignedIntegerInRange $input $begin $end]
    }
}

proc checkSignedIntegerInRange {input begin end} {
   if { [checkSignedInteger $input] == 0 } {
     return 0
   }

   if { $input < $begin || $input > $end } {
     return 0
   }
   return 1
}

#
# Check if unsigned integer is in the given range.
# 
# input is the number to be matched.
# begin is the minimum allowed value.
# end  is the maximum allowed value.
#
# Returns: 1 if input number is in range
#          0 on failure
#
proc checkUnsignedIntegerInRange {input begin end} {

    set ret 0
    if {[checkInteger $input]} {
        if {![catch {mpexpr $input > 0}] && ![catch {mpexpr $begin > 0}] &&
                  ![catch {mpexpr $end > 0}]} {
            if {[mpexpr $input >= $begin] && [mpexpr $input <= $end]} {
               set ret 1
            }
        }
    }
    return $ret
}


# Check if input is a year
proc checkYear { input } {
   if { [checkNonnegativeInteger $input] == 0 } {
     return 0
   }
   return [checkIntegerInRange $input 1970 2037]
}


# Check if input is a month
proc checkMonth  { input  } {
   set ret [checkIntegerInRange $input 1 12]
   return $ret
}


#Check if input is a day
proc checkDay { input  } {
   set ret [checkIntegerInRange $input 1 31]
   return $ret
}

# Return the number of days in Feb when year is the input
# returns -1 on error
proc daysInFebruary { year } {
  if { [checkYear $year] == 0 } {
     return -1
  }
  if { [ expr $year % 4] == 0 } {
     if { [ expr $year % 100 ] == 0 } {
        if { [ expr $year % 400 ] == 0 } { 
           return 29
        } else {
           return 28
        }
     } else {
        return 29
     }
  } else {
      return 28
  }
}


# Check if the inputs form a date
proc checkDate { year month day } {
  if { [checkYear $year] == 0 ||  [checkMonth $month] == 0 || [checkDay $day] == 0 } {
    return 0
  }

  set day1 [expr $day + 0]
  set month1 [expr $month + 0]

  #set array for days in a month
  set daysInMonth(1) 31
  set daysInMonth(2) 29
  set daysInMonth(3) 31
  set daysInMonth(4) 30
  set daysInMonth(5) 31
  set daysInMonth(6) 30
  set daysInMonth(7) 31
  set daysInMonth(8) 31
  set daysInMonth(9) 30
  set daysInMonth(10) 31
  set daysInMonth(11) 30
  set daysInMonth(12) 31

  if { $day1 > $daysInMonth($month1) } {
    return 0
  }
  if { $month1 == 2 && $day1 > [daysInFebruary $year] } {
    return 0
  }
  return 1
}

# Check if input is IPv4 multicast address
proc checkIPv4_Mcast { input } {
   set sub(0) 0
   set ret [ regexp {(^[0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+$)} $input match sub(1) sub(2) sub(3) sub(4) ]
   if { $ret == 0 } {
      return $ret
   }

   set c0 0
   set c255 0
   for {set i 1} {$i < 5} {incr i} {
       if {[regexp {^0[0-9]+} $sub($i)] == 1 || $sub($i) > 255}  {
          return 0
       }
       if {$sub($i) == 0}   {incr c0}
       if {$sub($i) == 255} {incr c255}
   }
   if {$c0 == 4 || $c255 == 4} {
      return 0
   }

   if { $sub(1) == 127 } {
      return 0
   }

   if { $sub(1) >= 224 && $sub(1) < 240 } {
      return 1
   }
   return 0
}

# Check if input is IPv4 unicast address
proc checkIPv4_Unicast { input } {
   set sub(0) 0
   set ret [ regexp {(^[0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+$)} $input \
                                    match sub(1) sub(2) sub(3) sub(4)]
   if { $ret == 0 } {
      return $ret
   }

   set c255 0

   for {set i 1} {$i < 5} {incr i} {
       if {$sub($i) > 255 } {
           return 0
       }
       if {$sub($i) == 255} {incr c255}
   }

   if {$c255 == 4} {
       return 0
   }

   if { $sub(1) > 223 && $sub(1) < 240 } {
      return 0
   }

   return 1
}

# Check if input is IPv4 address
proc checkIPv4 { string } {
  set sub(0) 0
  set ret [ regexp {(^[0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+$)} $string match sub(1) sub(2) sub(3) sub(4) ]

  if { $ret == 1 } {
      for {set i 1} {$i < 5} {incr i} {
          if {[regexp {^0[0-9]+} $sub($i)] == 1 || $sub($i) > 255 } {
              return 0
          }
      }
  }
  return $ret
}
 
# Checks if the given string has the length less or equal to the input
# length
proc checkStrLen { input_string len } {
   if { [string length $input_string] <= $len } {
       return 1
   } else {
       return 0
   }
}

# Check if the input string has the length atleast equal to len or not
# returns 1 if true, 0 otherwise 
proc checkStrMinLen { input_string len } {
   if { [string length $input_string] >= $len } {
        return 1
   } else {
        return 0
   }
}

#
# Given a string verifies if it looks like hexadecimail part of IPv6
# address. 
# Accepts a string which might be:
#	 pure IPv6 address (without ::, zero compressor and no IPv4 embedded address)
#     OR just the hexadecimal part of the address.
# Returns a list of 2 elements
#	1st element is equal to 1 if it was a valid IPv6 address part, 0 otherwise 
#	2nd element is the number of the : sepereated IPv6 address pieces
#	
# Note: 2nd element of the list makes sense only if the 1st element is equal to 1
#
proc v6piece_check {input } {
    set ret_list [list]
    set in_list [split $input :]
    foreach elem $in_list {
       set ret [regexp {^([0-9a-fA-F]){1,4}$} $elem]
       if {$ret == 0} {
           lappend ret_list 0 0
           return $ret_list
       }
    }
    set in_len [llength $in_list]
    lappend ret_list 1 $in_len
    return $ret_list
}

    

#Check the IPv6 address - if it looks valid or not.
proc checkIPv6 { input } {
    #a regular exp. to check for invalid chars
    set ret [regexp {^[0-9a-fA-F:\.]*$} $input]
    if {$ret == 0} { return 0 }

    #though an unspecified address it required to be allowed
    if {$input == "::"} {return 1}

    set expand [string first "::" $input]
    if {$expand >= 0} {
        # Now check for multiple "::"
        set empty 0
        set lst [split $input :]
        foreach elem $lst {
            if {$elem == ""} {
                incr empty
            }
        }
        if {$expand == 0 || [string range $input end-1 end] == "::"} {
            #special case, where the addr begins or ends with "::"
            if {$empty > 2} {
                return 0
            }
        } elseif {$empty > 1} {
            return 0
        }
    }

    # Check for IPv6 and IPv4 mixed addr.
    set mixed [regexp {^[0-9a-fA-F:]*:[0-9a-fA-F]*:(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$} $input]
    if {$mixed == 1} {
        set lst [split $input :]
        set ipv4_part [lindex $lst [expr [llength $lst] -1] ]
        if {[checkIPv4 $ipv4_part] == 0} { return 0 }
        #if it is ::205.226.2.101 type
        if {"::$ipv4_part" == "$input"} { return 1 }
    }

    set lst [split $input :]
    set len [llength $lst]
    # the shortest address can be ::1 or 1::
    if {$len < 3} { return 0 }

    if {$expand >= 0} {
        set prefix ""
	if {$expand > 0} {
            set prefix [string range $input 0 [expr $expand - 1]]
        }
        set prelen 0
        if {$prefix != ""} {
            set ret [v6piece_check $prefix]
            if {[lindex $ret 0] == 0} {return 0}
            set prelen [lindex $ret 1] 
        }
        
        set postfix [string range $input [expr $expand + 2] end]
        set postlen 0
        if {$postfix != ""} {
            if {$mixed == 1} {
                set postlist [split $postfix :]
                set postlist [lrange $postlist 0 end-1]
                set postfix [join $postlist :]
            }
            set ret [v6piece_check $postfix]
            if {[lindex $ret 0] == 0} {return 0}
            set postlen [lindex $ret 1] 
        }
        set total_len [expr $prelen + $postlen]
        if {$mixed == 1} {
            if {$total_len >= 6} { return 0 }
        } else {
            if {$total_len >= 8} { return 0 }
        }
        #All is fine now.
        return 1
    }

    #The IPv6 addr does not have "::" in it
    if {$mixed == 1} {
        if {$len != 7} {return 0}
        set lst [lrange $lst 0 end-1]
        set input2 [join $lst :]
        set ret [v6piece_check $input2]
        if {[lindex $ret 0] == 0} {return 0}
    } else {
        if {$len != 8} { return 0 }
        set ret [v6piece_check $input]
        if {[lindex $ret 0] == 0} {return 0}
    }

    return 1
}

#
# Proc - create_IPv6_piece
# 
# Accepts: one hexadicimal piece of IPv6 address 
#
# Returns: a piece suitable to be put it database
#	   - adds leading zeros
#	   - makes it a 4 digit hexadecimal number
#	   - makes it all in lower case
#
proc create_v6_piece {input } {
    # find out how much padding to be applied on this piece
    set pad [expr 4 - [string length $input]]
 
    # now create the padding
    set zeros [string repeat 0 $pad]

    #add the padding infront of it
    set input "$zeros$input"

    #change all the hex digits to lower case
    set input [string tolower $input]
    return $input
}

#
# Proc - d2x maps the input string to hex degits.
# 
# Accepts - a string and expects it to be between 10 and 15, both inclusive
#
# Returns - the corresponding hex digit, if input is not between 10 & 15, it
#           returns the input itself.
#
proc d2x {input} {
   return [string map {
                10   a
                11   b
                12   c
                13   d
                14   e
                15   f} $input]
}


# 
# Proc - v4tov6 convert IPv4 address to hexadicmal form
#
# Accepts - a string which represents the dotted decimal form of IPv4 address, it
#	    assumes the input is a correct IPv4 address, as checked by checkIPv4
#	    function.
#
# Returns - a string which is the last 8 charachters of the IPv6 address in 
#           hexadicmal form
#           the string is all in lower case
#           it is zero padded 
#
proc v4tov6 {input} {
    set lst [split $input .]
    set v6part "" 
    foreach elem $lst {
        if {$elem >= 16} {
	    # the reason we divide and mod only once, is the no part of IPv4 
	    # should be greater than 255, in which case the remainder would always
	    # be less than 16.
            set remain [d2x [expr $elem % 16]]
            set elem [d2x [expr $elem / 16]]
            set hex "$elem$remain"
        } else {
            set hex [d2x $elem]
        } 
        if {[string len $hex] < 2} {
            set hex "0$hex"
        }
        set v6part "$v6part$hex"
    }
    return $v6part
}

#
#proc expand IPv6 address suitable to be put in the database
# Accepts: a correct IPv6 address, it assumes that the input has been 
#          verified by checkIPv6, validation function
#
# Returns: a string which is the suitable string to go in the database.
#
# Suitable means:
#	Expand zero compression
#	Change IPv4 part to hexadecimal
#	Add leading zeros (making it 4 hexa-digits for each piece)
#
# eg. :		FF01:0:0:0:0:0:0:43    -> FF010000000000000000000000000043
#		FF01::43	       -> FF010000000000000000000000000043
#		0:0:0:0:0:0:13.1.68.3  -> 0000000000000000000000000D014403
#
proc expandIPv6 {input } {

    # check if the address has zero comprssed
    set expand [string first "::" $input]

    # check if its a IPv4 and IPv6 mixed address
    set mixed [regexp {^[0-9a-fA-F:]*:(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$} $input]

    set lst [split $input :]
    set ipv4_part ""
    if {$mixed == 1} {
        set ipv4_part [lindex $lst end]
        set ipv4_part [v4tov6 $ipv4_part]
    }

    set prefix ""
    set prelen 0
    set prelist ""
    if {$expand >= 0} {
        set prefix [string range $input 0 [expr $expand - 1]]
        set prelist [split $prefix :]
        set prelen [llength $prelist]
    }

    if {$expand >= 0} {
        set postfix [string range $input [expr $expand + 2] end]
    } else {
        set postfix $input
    }
    set postlist ""
    set postlen 0
    if {$postfix != ""} {
        if {$mixed == 1} {
            set postlist [split $postfix :]
            set postlist [lrange $postlist 0 end-1]
            set postfix [join $postlist :]
        } 
        set postlist [split $postfix :]
        set postlen [llength $postlist]
    }

    set pre_v6addr ""
    foreach elem $prelist {
        set ret [create_v6_piece $elem]
        set pre_v6addr "$pre_v6addr$ret"
    }
    set post_v6addr ""
    foreach elem $postlist {
        set ret [create_v6_piece $elem]
        set post_v6addr "$post_v6addr$ret"
    }
    if {$mixed == 1} {
        set post_v6addr "$post_v6addr$ipv4_part"
    }
    set pad [expr 32 - [expr [string length $pre_v6addr] + [string length $post_v6addr]]]
 
    set zeros [string repeat "0" $pad]

    return "$pre_v6addr$zeros$post_v6addr"
}


#
# proc v6db2display - takes in a string which represent the IPv6 address as 
#                     in database and converts it to displayable address.
#
# Accepts - a string represnting IPv6 address, as represented in the database
#
# Returns - a colon seperated IPv6 address, do compression of zeros if required.
#	    The longest contiguous string of zeros is compressed to ::
#
proc v6db2display {input} {
    set counter(0) 0
    for {set i 0} {$i < 8} {incr i} {
        set counter($i)  0
    }
    set index -1

    for {set i 0} {$i < 8} {incr i} {
        set j [expr $i * 4]
        set ret [string trimleft [string range $input $j [expr $j + 3]]  0]
        if {$ret == ""} {
            set ret 0
            if {$index < 0} {set index $i}
            set counter($index) [expr $counter($index) + 1]
        } else {
            set index -1
        }
        set address($i) $ret
    }

   # find the longest contiguous string of zeros
    set max 1
    set max_index -1
    for {set i 0} {$i < 8} {incr i} {
        if {$counter($i) > $max} {
            set max $counter($i)
            set max_index $i
        }
    }

    # Now create the address
    set v6addr ""
    for {set i 0} {$i < 7} {incr i} {
        if {$max_index >= 0 && $max_index == $i} {
            if {$max_index == 0} {
                set v6addr "::"
            } else {
                set v6addr "$v6addr:"
            }
            incr i [expr $max - 1]
        } else {
            set v6addr "$v6addr$address($i):"
        }
    }
    if {$i < 8} {
        set v6addr "$v6addr$address(7)"
    }
    return $v6addr
}

#
# proc v6dot2disp
#
# Accepts - a string as returned by ipsctl, representing a IPv6 address
#	    12ab.0.ff.ab3f.0.0.0.ffbc
#
# Returns - a displayable IPv6 address, exactly as returned by v6db2display
#           function
#
proc v6dot2disp { input } {
    set expand [string first ".." $input]
    set prefix ""
    set postfix ""
    if {$expand >= 0} {
        set prefix [string range $input 0 [expr $expand - 1]]
        set postfix [string range $input [expr $expand + 2] end]
    } else {
        set postfix $input
    }
    set prelst [split $prefix .]
    set prelen [llength $prelst]
    set postlst [split $postfix .]
    set postlen [llength $postlst]
    set padlen [expr 8 - [expr $prelen + $postlen]]
    
    set v6addr ""
    foreach elem $prelst {
        set pl [expr 4 - [string length $elem]]
        set elem "[string repeat 0 $pl]$elem"
        set v6addr "$v6addr$elem"
    }
  
    set v6addr "$v6addr[string repeat 0 [expr $padlen * 4]]"
    foreach elem $postlst {
        set pl [expr 4 - [string length $elem]]
        set elem "[string repeat 0 $pl]$elem"
        set v6addr "$v6addr$elem"
    }

    return [v6db2display $v6addr]
}


#
# proc Validate - takes the Q_Data array of variable, value and type
# 		  and performs the type checking.
# 		  One more thing that it validates is, if mandatory 
#		  fields are filled or not.
# what it does not do is, checking of variable like combo did, that is
# depending on value of one variable, check the value of other.
# 
# The second argument is optional, if given its filled with error
# messages if any.
# It is not necessary for the 2nd variable to exist in the scope of your
# script, the function will create it if it does not already exist, and 
# set to the error list
#
proc Validate { Q_Data  {err_list ""} } {

    set temp_list [list]
    set llen [llength $Q_Data]
    set mand_err 0

    for {set i 0} {$i < $llen} {incr i 3} {
        set value [lindex $Q_Data [expr $i + 1]]
        set type [lindex $Q_Data [expr $i + 2]]
        set tlist [split $type :]

        # check if the field was required or not.
        # emptyOK == 0, means the field was required, set error string
        # if the field was left empty.
        set emptyOK [lindex $tlist 1]
        if {$emptyOK == 0 && $value == ""} {
            if {$mand_err == 0} { set mand_err 1 }
            continue
        }

        # the field was optional, so let's just process the next entry
        if {$emptyOK == 1 && $value == ""} {continue}
        
        switch -exact -- [lindex $tlist 0] {
            INT {
                if {[checkInteger $value] == 0} {
                    lappend temp_list "Invalid number: $value. Integers must be maximum: 4294967295, minimum: -2147483647"
                }
            }
            INT_R {
                set min [lindex $tlist 2]
                set max [lindex $tlist 3]
                if {[checkIntegerInRange $value $min $max] == 0} {
                    lappend temp_list "Invalid number: $value. Not in range $min-$max."
                }
            }
            INT_N {
                if {[checkNegativeInteger $value] == 0} {
                    lappend temp_list "Invalid number: $value. Negative integers must be maximum: -1, minimum -2147483647."
                }
            }
            INT_P {
                if {[checkPositiveInteger $value] == 0} {
                    lappend temp_list "Invalid number: $value. Positive integers must be maximum: 4294967295, minimum: 1"
                }
            }
            MAC {
                if {[checkMACAddr $value] == 0} {
                    lappend temp_list "Invalid MAC address: $value"
                }
            }
            IP6 {
                if {[checkIPv6 $value] == 0} {
                    lappend temp_list "Invalid IPv6 address: $value"
                }
            }
            IP4 {
                if {[checkIPv4 $value] == 0} {
                    lappend temp_list "Invalid IPv4 address: $value"
                }
            }
            IP4U {
                if {[checkIPv4_Unicast $value] == 0} {
                    lappend temp_list "Invalid IPv4 unicast address: $value"
                }
            }
            IP4M {
                if {[checkIPv4_Mcast $value] == 0} {
                    lappend temp_list "Invalid IPv4 multicast address: $value"
                }
            }
            PASS { }
            EXPR {
	        set rexpr [lindex $tlist 2]
		set emsg  [lindex $tlist 3]
		if {[checkUserDef $value $rexpr] == 0} {
                    if {$emsg == ""} {
                        lappend temp_list "$value: Does not satisfy regular expression \'$expr\'."
		    } else {
                        lappend temp_list "$value: $emsg"
		    }
		}
            }
            STR  { }
            STR_L {
                set minlen [lindex $tlist 2]
                if {[checkStrLen $value $minlen] == 0} {
                    lappend temp_list "Maximum allowed string length is $minlen."
                }
            }
            HOST {
                if {[checkHostName $value] == 0} {
                    lappend temp_list "Invalid hostname: $value"
                }
            }
            EMAIL {
                if {[checkEmail $value] == 0} {
                    lappend temp_list "Invalid email address: $value"
                }
            }
            URL {
                if {[checkURL $value] == 0} {
                    lappend temp_list "Invalid URL: $value"
                }
            }
            FILE { }
            default {
#               puts "should not happen.'
            }
        }
    }
    set return_code 0
    if {$mand_err == 1} {
        lappend temp_list "Not all fields were completed."
    }
    if {[llength $temp_list] > 0} {
        incr return_code
    }
    if {$err_list != ""} {
        foreach msg $temp_list {
            uplevel 1 [list lappend $err_list $msg]
        }
    }
    return $return_code
}




###########################################################
# proc check_valid_slot
# checks valid slot
# checks for range and connectivity
# returns err_list 
############################################################

proc check_valid_slot {  {slot_num  ""} ipsctl_handle  \
   { check_connected "1" } } {
   global err_list
   if { $slot_num == "" } {
      lappend err_list " slot num = NULL "
      return 1
   } elseif {  ![checkPositiveInteger $slot_num ] } {
      lappend err_list " \[$slot_num\] slot should be a positive integer  "
      return 1
   } else {
      set max_slots [ ipsctl get $ipsctl_handle  hw:chassis:linecards ]
      if { $slot_num > $max_slots  || $slot_num <= 0 } {
         lappend err_list  " slot $slot_num Not is Range \[1-$max_slots\] "
         return 1
      } else {
         set slot_connected [ipsctl get $ipsctl_handle \
         daemon:slotman:slot:$slot_num:present ]
         if { $check_connected == 1 &&  $slot_connected == 0  } {
            lappend err_list  "slot  $slot_num is Not Connected "
            return 1
         }
      }
   }
return  0
}

#********************************************************************
# Procedure   : checkIPv4_against_masklen
# Description : checks an IP address for host bits set in mask positions
#		set to 0
# Arguments   : IPaddr  -> the IP address
#err		masklen -> the mask length
# Returns     : 0: failed
#		1: passed
# See also    :
#*******************************************************************/

proc checkIPv4_against_masklen { IPaddr masklen } {
  if { $masklen > 32 } {
      return 0
  } elseif { $masklen == 32 } {
    return 1
  }

  set ret [ regexp {(^[0-9]+).([0-9]+).([0-9]+).([0-9]+$)} \
	     $IPaddr match sub1 sub2 sub3 sub4 ]
  if { $ret == 0 } {
      return 0
  }

  set lcl_msg_list [list]
  set IP_as_int [ expr ($sub1 << 24) + ($sub2 << 16) + ($sub3 << 8) + $sub4 ]
  if { $IP_as_int == 0 } {
    return 0
  }

  set mask [expr (0x7FFFFFFF >> $masklen) + (1 << (32 - $masklen - 1))]
  set res [expr $IP_as_int & $mask]
  if { $res == 0 } {
    return 1
  } else {
    return 0
  }
}

#********************************************************************
# Procedure   : append_gplc
# Description : generates additional db bindings for the GPLCs (Sequoia)
# Arguments   : set_list name; ipsctl handle; binding; value
# Returns     : 0: failed
#               1: passed
#*******************************************************************/

proc append_gplc { set_list ipsctl_handle binding value} {
        upvar $set_list set_list_local
        set is_crp [ipsctl get $ipsctl_handle hw:motherboard:crp]
        if { $is_crp != 1 } {
                return
        }
        set max_slots 0
        set max_slots [ ipsctl get $ipsctl_handle  hw:chassis:linecards ]
        if { $max_slots <= 0 } {
                return
        }
        for {set i 1} {$i <= $max_slots} {incr i} {
                set slotman_conn 0
                set slotman_conn [ipsctl get $ipsctl_handle \
                        daemon:slotman:slot:$i:present ]
                set slot_connected 0
                set slot_connected [ipsctl get $ipsctl_handle \
                        hw:chassis:slot:$i:link ]
                set slot_link 0
                set slot_link [ipsctl get $ipsctl_handle \
                        hw:chassis:slot:$i:present ]
                if { $slot_connected == 1 && $slotman_conn == 1 && $slot_link == 1 } {
                        if {[string index $binding 0] == ":"} {
                                lappend set_list_local :node:$i$binding $value
                        } else {
                                lappend set_list_local node:$i:$binding $value
                        } 
                }
        }
        return 1
} 


# check is given path is absolute
# 1 : good (absolute)
# 0: bad (not absolute)
proc checkAbsolutePath { name } {
  set cf_retVal 1

  if { [string compare [string index $name 0] "/"] != 0 } {
      set cf_retVal 0
  }
  return $cf_retVal
}

# check is given file name is good name
# 1 : good 
# 0: bad 
proc checkGoodFileName { name } {
  set cf_retVal 1

  if { [regexp {^[a-zA-Z0-9_\-\/\.]+$} $name ] == 0 } {
      set cf_retVal 0
  }
  if { [string compare [string index $name end] "/"] == 0 } {
      set cf_retVal 0
  }
  return $cf_retVal
}

# check is given dir name is good name
# 1 : good 
# 0: bad 
proc checkGoodDirName { name } {
  set cf_retVal 1

  if { [regexp {^[a-zA-Z0-9_\-\/\.]+$} $name ] == 0 } {
      set cf_retVal 0
  }

  return $cf_retVal
}
